//
//  AMSDKLogger.h
//  AliMeetingSDK
//
//  Created by xuanzhen.zjw on 2019/8/15.
//  Copyright © 2019 Alibaba. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@class AMSDKLogger;

/**
 * 日志标识
 */
typedef NS_ENUM(NSInteger, AMSDKLoggingFlag) {
    //! Error
    AMSDKLoggingFlagError      =  (1 << 0),
    
    //! Warning
    AMSDKLoggingFlagWarning    =  (1 << 1),
    
    //! Info
    AMSDKLoggingFlagInfo       =  (1 << 2),
    
    //! Debug
    AMSDKLoggingFlagDebug      =  (1 << 3),
    
    //! Verbose
    AMSDKLoggingFlagVerbose    =  (1 << 4),
};

/**
 *  日志级别
 */
typedef NS_ENUM(NSUInteger, AMSDKLoggingLevel){
    //! 关闭
    AMSDKLoggingLevelOff       = 0,
    
    //! 输出Error日志
    AMSDKLoggingLevelError     = (AMSDKLoggingFlagError),
    
    //! 输出Error、Warning日志
    AMSDKLoggingLevelWarning   = (AMSDKLoggingLevelError   | AMSDKLoggingFlagWarning),
    
    //! 输出Error、Warning、Info日志
    AMSDKLoggingLevelInfo      = (AMSDKLoggingLevelWarning | AMSDKLoggingFlagInfo),
    
    //! 输出Error、Warning、Info、Debug日志
    AMSDKLoggingLevelDebug     = (AMSDKLoggingLevelInfo    | AMSDKLoggingFlagDebug),
    
    //! 输出Error、Warning、Info、Debug、Verbose日志
    AMSDKLoggingLevelVerbose   = (AMSDKLoggingLevelDebug   | AMSDKLoggingFlagVerbose),
    
    //! 输出所有日志
    AMSDKLoggingLevelAll       = NSUIntegerMax
};

/**
 * 会议日志协议，可实现此协议处理会议日志信息
 */
@protocol AMSDKLoggerProtocol <NSObject>

@required

//! 输出error级别日志
+ (void)error:(NSString *)tag msg:(NSString *)msg;

//! 输出warn级别日志
+ (void)warn:(NSString *)tag msg:(NSString *)msg;

//! 输出info级别日志
+ (void)info:(NSString *)tag msg:(NSString *)msg;

//! 输出debug级别日志
+ (void)debug:(NSString *)tag msg:(NSString *)msg;

//! 输出verbose级别日志
+ (void)verbose:(NSString *)tag msg:(NSString *)msg;

//! SLS Action日志
+ (void)action:(NSString *)module
         point:(NSString *)monitorPoint
        params:(NSDictionary * _Nullable)params;

//! SLS Event日志
+ (void)event:(NSString *)module
        point:(NSString *)monitorPoint
       params:(NSDictionary * _Nullable)params;

//! SLS Error日志
+ (void)error:(NSString *)module
        point:(NSString *)monitorPoint
       params:(NSDictionary * _Nullable)params;

@end

/**
 * 默认Logger实现，如果没有注入日志实现类则调用NSLog
 */
@interface AMSDKLogger : NSObject <AMSDKLoggerProtocol>

//! 日志等级
@property (nonatomic, class) AMSDKLoggingLevel level;

//! 注入日志类
+ (void)setLoggerPrinter:(Class<AMSDKLoggerProtocol>)printer;

@end

#ifdef  __cplusplus
extern "C" {
#endif
    //! 获取当前日志所处文件名
    extern NSString* AMSDKLoggingFileName(const char *fileName);
    //! 输出日志方法
    extern void AMSDKLogging(AMSDKLoggingFlag flag, NSString *tag, NSString *string);
    
#ifdef  __cplusplus
}
#endif

#define AMSDKLoggingString(format, ...)                         \
[NSString stringWithFormat:@"(%@:%d %s): " format,              \
AMSDKLoggingFileName(__FILE__),                                 \
__LINE__,                                                       \
__FUNCTION__,                                                   \
##__VA_ARGS__                                                   \
]

#define AMSDKLoggingFormat(flag, tag, format, ...)                   \
do {                                                                 \
AMSDKLogging(flag, tag, AMSDKLoggingString(format, ##__VA_ARGS__));  \
} while(NO)

#define AMSDKLoggingVerbose(tag, format, ...)                   \
AMSDKLoggingFormat(AMSDKLoggingFlagVerbose, tag, format, ##__VA_ARGS__)

#define AMSDKLoggingInfo(tag, format, ...)                      \
AMSDKLoggingFormat(AMSDKLoggingFlagInfo, tag, format, ##__VA_ARGS__)

#define AMSDKLoggingWarn(tag, format, ...)                      \
AMSDKLoggingFormat(AMSDKLoggingFlagWarning, tag, format, ##__VA_ARGS__)

#define AMSDKLoggingError(tag, format, ...)                     \
AMSDKLoggingFormat(AMSDKLoggingFlagError, tag, format, ##__VA_ARGS__)

#define AMSDKLoggingDebug(tag, format, ...)                     \
AMSDKLoggingFormat(AMSDKLoggingFlagDebug, tag, format, ##__VA_ARGS__)

NS_ASSUME_NONNULL_END
